;
; Copyright (C) 2006 Johannes Hausensteiner (johannes.hausensteiner@pcl.at)
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
;
;
; Filename: diy_calc.asm
;
; Function: ROM monitor for the DIY Calculator
;
;
; assemble with tasm with following parameters:
;   tasm -01 -i -o10 -c -f0 diy_calc.asm diy_calc.hex
;
; generate the .mem file for import into ispLEVER FPGA project
;   hex2mem diy_calc
;
;
; Changelog
;
;  0.1    6.Sep.2006   JH   new
;
;

#include "diy_calc.inc"
#include "monitor.inc"

VERSION:   .EQU    $01

MON_RAM    .EQU    $FE00          ; special RAM area for the ROM monitor

CMD_HELP   .EQU    $00            ; display help
CMD_VER    .EQU    $01            ; display version information
CMD_MRD    .EQU    $02            ; memory read cmd
CMD_MWR    .EQU    $03            ; memory write cmd
CMD_LOAD   .EQU    $04            ; load a program
CMD_RUN    .EQU    $05            ; run the program
CMD_TEST   .EQU    $06            ; for tests

BEL        .EQU    $07            ; bell
BS         .EQU    $08            ; backspace
CR         .EQU    $0A            ; carriage return
LF         .EQU    $0D            ; line feed

           .ORG    MON_RAM
PUTCHAR    .DW                    ; pointer to putc() function
PUTS       .DW                    ; pointer to puts() function
TSTC       .DW                    ; pointer ot tstc() function
GETCH      .DW                    ; pointer to getch() function
GETCHE     .DW                    ; pointer to getche() function
GETCHAR    .DW                    ; pointer to getchar() function
GETS       .DW                    ; pointer to gets() function
PRINTF     .DW                    ; pointer to printf() function
MALLOC     .DW                    ; pointer to malloc() function
FREE       .DW                    ; pointer to free() function

; general use
PTR16:     .DW                    ; 16bit pointer
TMP16:     .DW                    ; temp 16bit memory
TMP:       .DB
ERROR:     .DB
BUF        .BLOCK  40

; use in main()
CMDLINE:   .BLOCK  40             ; reserve a buffer of 40 bytes
CMDSTR:    .BLOCK  8              ; the cmd extracted from cmdline
CMD:       .DB

; use in strcmp()
R_ADD:     .DW
S1:        .DW
S2:        .DW

; use in md()
MD_START:  .DW                    ; start address of memory region (md cmd)
MD_END:    .DW                    ; end address of memory region (md cmd)
MD_ADDR:   .DW                    ; current address of memory region (md cmd)
MD_NUM:    .DW                    ; number of bytes to write/display (md cmd)
MD_OPT:    .DW                    ; ptr to current cmdline option (md cmd)

; use in mw()
MW_START:  .DW                    ; start address of memory region (mw cmd)
MW_END:    .DW                    ; end address of memory region (mw cmd)
MW_ADDR:   .DW                    ; current address of memory region (mw cmd)
MW_NUM:    .DW                    ; number of bytes to write/display (mw cmd)
MW_OPT:    .DW                    ; ptr to current cmdline option (mw cmd)
MW_DATA:   .DB                    ; write data

; use in load()
BYTECNT    .DB
ADDR       .DW
TYPE       .DB
DATA       .DB
SUM        .DB
CHECKSUM   .DB


           .ORG    $0000
RESET:
; do some initialisations
           LDA     [IRQ_SRC]
           AND     IRQ_COLD
           JZ      [INIT]
           INCA
           JMP     [MAIN]

INIT:      BLDSP   RAMEND
           BLDX    MSG001
           JSR     [_PUTS]
           LDA     0
           STA     [CMD]
           BLDX    $4000
           BSTX    [MD_START]
           BLDX    $0100
           BSTX    [MD_NUM]
           BLDX    $4000
           BSTX    [MW_START]
           BLDX    1
           BSTX    [MW_NUM]
           LDA     CLRCODE
           JSR     [LCDPUT]
           BLDX    MSG003
           JSR     [LCDPUTS]
           VER
           JSR     [LCDPUTH]

; assign function pointers
           BLDX    _PUTCHAR
           BSTX    [PUTCHAR]
           BLDX    _PUTS
           BSTX    [PUTS]
           BLDX    _TSTC
           BSTX    [TSTC]
           BLDX    _GETCH
           BSTX    [GETCH]
           BLDX    _GETCHE
           BSTX    [GETCHE]
           BLDX    _GETCHAR
           BSTX    [GETCHAR]
           BLDX    _GETS
           BSTX    [GETS]
           BLDX    _NOTYET
           BSTX    [PRINTF]
           BLDX    _NOTYET
           BSTX    [MALLOC]
           BLDX    _NOTYET
           BSTX    [FREE]

; print welcome message
           BLDX    MSG002
           JSR     [puts]
           JSR     [PUTVER]
           JSR     [newline]

; mainloop
MAIN:      JSR     [PROMPT]
; read the command
           BLDX    CMDLINE
           JSR     [gets]

; extract the command from the cmdline
           BLDX    CMDLINE
           BSTX    [TMP16]
           BLDX    CMDSTR
           BSTX    [PTR16]
           BLDX    0
LBL009:    LDA     [[TMP16], X]
           STA     [[PTR16], X]
           JZ      [LBL010]       ; no cmdline option
           INCX
           CMPA    ' '
           JNZ     [LBL009]
           DECX
LBL010:    INCX
           DECX
           JZ      [LBL011]       ; cmdline was empty => exec previous cmd
           LDA     0              ; 0-terminate the command
           STA     [[PTR16], X]

; analyze the command
           BLDX    CMDSTR
           PUSHX
           BLDX    CMDSTR_HELP
           PUSHX
           JSR     [STRCMP]
           JNZ     [LBL002]
           LDA     CMD_HELP       ; "help" cmd
           JMP     [LBL001]

LBL002:    BLDX    CMDSTR
           PUSHX
           BLDX    CMDSTR_VER
           PUSHX
           JSR     [STRCMP]
           JNZ     [LBL003]
           LDA     CMD_VER        ; "ver" cmd
           JMP     [LBL001]

LBL003:    BLDX    CMDSTR
           PUSHX
           BLDX    CMDSTR_MRD
           PUSHX
           JSR     [STRCMP]
           JNZ     [LBL004]
           BLDX    CMDLINE
           LDA     CMD_MRD        ; "md" cmd
           JMP     [LBL001]

LBL004:    BLDX    CMDSTR
           PUSHX
           BLDX    CMDSTR_MWR
           PUSHX
           JSR     [STRCMP]
           JNZ     [LBL005]
           BLDX    CMDLINE
           LDA     CMD_MWR        ; "mw" cmd
           JMP     [LBL001]

LBL005:    BLDX    CMDSTR
           PUSHX
           BLDX    CMDSTR_LOAD
           PUSHX
           JSR     [STRCMP]
           JNZ     [LBL006]
           LDA     CMD_LOAD       ; "load" cmd
           JMP     [LBL001]

LBL006:    BLDX    CMDSTR
           PUSHX
           BLDX    CMDSTR_RUN
           PUSHX
           JSR     [STRCMP]
           JNZ     [LBL007]
           LDA     CMD_RUN        ; "run" cmd
           JMP     [LBL001]

LBL007:    BLDX    CMDSTR
           PUSHX
           BLDX    CMDSTR_TEST
           PUSHX
           JSR     [STRCMP]
           JNZ     [LBL008]
           LDA     CMD_TEST       ; "test" cmd
           JMP     [LBL001]

LBL008:    BLDX    CMDSTR_UNKNOWN
           JSR     [puts]
           LDA     $FF
           STA     [CMD]
           JMP     [MAIN]

LBL001:    STA     [CMD]

; build jump table
LBL011:    LDA     0
           STA     [TMP16]
           LDA     [CMD]
           SHL
           SHL
           STA     [TMP16+1]
           BLDX    [TMP16]
           JSR     [CMD_TABLE, X]
           JMP     [MAIN]

; cmd jump table, each entry must be 4 bytes
CMD_TABLE: JMP     [HELP_CMD]
           NOP

           JMP     [VER_CMD]
           NOP

           JMP     [MD_CMD]
           NOP

           JMP     [MW_CMD]
           NOP

           JMP     [LOAD_CMD]
           NOP

           JMP     [RUN_CMD]
           NOP

           JMP     [TEST_CMD]
           NOP

CMDSTR_HELP: .BYTE "help", 0      ; display help
CMDSTR_VER:  .BYTE "ver", 0       ; display version information
CMDSTR_MRD:  .BYTE "md", 0        ; memory display cmd
CMDSTR_MWR:  .BYTE "mw", 0        ; memory write cmd
CMDSTR_LOAD: .BYTE "load", 0      ; load program cmd
CMDSTR_RUN:  .BYTE "run", 0       ; run program cmd
CMDSTR_TEST: .BYTE "test", 0      ; for test
CMDSTR_UNKNOWN: .BYTE "Unknown command - try \"help\"", CR, LF, 0

MSG001:    .BYTE   "DIY Calculator ", 0
MSG002:    .BYTE   "ROM monitor V", 0
MSG003:    .BYTE   "CPU Rev: ", 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function putver: print the ROM monitor version in x.y format
;
; Input:  --
;
; Output: --
;
; Uses:   A, C, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PUTVER:    LDA     VERSION
; process high nibble
           SHR
           SHR
           SHR
           SHR
           AND     $0F
           CMPA    9
           JC      [PUTVER1]
           ADD     '0'
           JMP     [PUTVER2]
PUTVER1:   ADD     'A'-10
PUTVER2:   JSR     [putchar]
           LDA     '.'
           JSR     [putchar]
; process low nibble
           LDA     VERSION
           AND     $0F
           CMPA    9
           JC      [PUTVER3]
           ADD     '0'
           JMP     [PUTVER4]
PUTVER3:   ADD     'A'-10
PUTVER4:   JSR     [putchar]
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function help_cmd: execute the help cmd
;
; Input:  --
;
; Output: --
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
HELP_CMD:  BLDX    HELP_CMD1
           JSR     [puts]
           BLDX    HELP_CMD2
           JSR     [puts]
           BLDX    HELP_CMD3
           JSR     [puts]
           BLDX    HELP_CMD4
           JSR     [puts]
           BLDX    HELP_CMD5
           JSR     [puts]
           BLDX    HELP_CMD6
           JSR     [puts]
           BLDX    HELP_CMD7
           JSR     [puts]
           RTS

HELP_CMD1: .BYTE   CR, LF, "Following commands are available:", CR, LF, 0
HELP_CMD2: .BYTE   "  help .. this text", CR, LF, 0
HELP_CMD3: .BYTE   "  ver ... display version information", CR, LF, 0
HELP_CMD4: .BYTE   "  md .... memory display", CR, LF, 0
HELP_CMD5: .BYTE   "  mw .... memory write", CR, LF, 0
HELP_CMD6: .BYTE   "  load .. load a program (Intel HEX format)", CR, LF, 0
HELP_CMD7: .BYTE   "  run ... run the program at address $4000", CR, LF, 0
HELP_CMD8: .BYTE   "  test .. do some tests", CR, LF, 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function ver_cmd: execute the version cmd
;
; Input:  --
;
; Output: --
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
VER_CMD:   BLDX    VER_CMD1
           JSR     [puts]
           LDA     [HW_REV]
           JSR     [PUTHEX]
           BLDX    VER_CMD2
           JSR     [puts]
           VER
           JSR     [PUTHEX]
           JSR     [newline]
           RTS

VER_CMD1:  .BYTE   CR, LF, "FPGA version: $", 0
VER_CMD2:  .BYTE   CR, LF, "CPU  version: $", 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function md_cmd: execute the memory display cmd:
;                  md address [; of bytes]
;
; Input:  X ... ptr to cmdline
;
; Output: X .. next address
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
MD_CMD:    BLDX    MD_MSG1
           JSR     [puts]

; extract cmdline parameters
           BLDX    CMDLINE
           BSTX    [MD_OPT]
; skip the cmd itself
MD_CMD1:   LDA     [[MD_OPT]]
           JZ      [MD_CMD2]      ; end of string, no parameters
           BLDX    [MD_OPT]
           INCX
           BSTX    [MD_OPT]
           CMPA    ' '
           JNZ     [MD_CMD1]

; call strcpy()
           BLDX    BUF
           PUSHX
           BLDX    [MD_OPT]
           PUSHX
           JSR     [STRCPY]

; convert string to number
           BLDX    BUF
           JSR     [ATOLX]
           BSTX    [MD_START]

; get the number of bytes
MD_CMD3:   LDA     [[MD_OPT]]
           JZ      [MD_CMD2]      ; end of string, no parameters
           BLDX    [MD_OPT]
           INCX
           BSTX    [MD_OPT]
           CMPA    ' '
           JNZ     [MD_CMD3]

; call strcpy()
           BLDX    BUF
           PUSHX
           BLDX    [MD_OPT]
           PUSHX
           JSR     [STRCPY]

; convert string to number
           BLDX    BUF
           JSR     [ATOLX]
           BSTX    [MD_NUM]

; compute end address
MD_CMD2:   LDA     [MD_START+1]
           ADD     [MD_NUM+1]
           STA     [MD_END+1]
           LDA     [MD_START]
           ADDC    [MD_NUM]
           STA     [MD_END]

; print start + end address
           LDA     [MD_START]
           JSR     [PUTHEX]
           LDA     [MD_START+1]
           JSR     [PUTHEX]
           BLDX    MD_MSG2
           JSR     [puts]
           BLDX    [MD_END]
           DECX
           BSTX    [PTR16]
           LDA     [PTR16]
           JSR     [PUTHEX]
           LDA     [PTR16+1]
           JSR     [PUTHEX]
           JSR     [newline]

; display address
MD_CMD6:   BLDX    [MD_START]
           BSTX    [MD_ADDR]
           BLDX    0
           LDA     [MD_START]
           JSR     [PUTHEX]
           LDA     [MD_START+1]
           JSR     [PUTHEX]

           LDA     ' '
           JSR     [putchar]

; display data
MD_CMD7:   LDA     ' '
           JSR     [putchar]
           LDA     [[MD_ADDR]]
           JSR     [PUTHEX]
; increment and check if last
           INCX
           BSTX    [TMP16]
           BLDX    [MD_ADDR]
           INCX
           BSTX    [MD_ADDR]
; compare high byte
           LDA     [MD_ADDR]
           CMPA    [MD_END]
           JC      [MD_CMD9]      ; current addr > end addr => finished
           JNZ     [MD_CMD8]      ; current < end => continue
; compare low byte
           LDA     [MD_ADDR+1]
           CMPA    [MD_END+1]
           JC      [MD_CMD9]      ; start > end => finished
           JZ      [MD_CMD9]      ; start == end => finished

; line done after 16 bytes
MD_CMD8:   LDA     [TMP16+1]
           AND     $0F
           JZ      [MD_CMD11]     ; done with HEX => continue with ASCII

; insert additional blank after 8 bytes
           AND     $07
           JNZ     [MD_CMD7]      ; next byte
           LDA     ' '
           JSR     [putchar]
           JMP     [MD_CMD7]      ; next byte

; we reached the end of display area => fill HEX line with ' '
MD_CMD9:
; have we reached the end of line (16 HEX bytes) already?
           LDA     [TMP16+1]
           AND     $0F
           JZ      [MD_CMD11]     ; yes => continue with ASCII
; insert additional blank after 8 bytes
           AND     $07
           JNZ     [MD_CMD17]
           LDA     ' '
           JSR     [putchar]
MD_CMD17:  LDA     ' '
           JSR     [putchar]      ; one for the ' '
           LDA     ' '
           JSR     [putchar]      ; one for the high nibble
           LDA     ' '
           JSR     [putchar]      ; one for the low nibble
           INCX
           BSTX    [TMP16]
           JMP     [MD_CMD9]      ; loop until 16 "bytes" filled

; display ASCII
MD_CMD11:  LDA     ' '
           JSR     [putchar]
           LDA     ' '
           JSR     [putchar]
           LDA     ' '
           JSR     [putchar]
           BLDX    [MD_START]
           BSTX    [MD_ADDR]
           BLDX    0
MD_CMD12:  LDA     [[MD_ADDR]]
           CMPA    $20
           JC      [MD_CMD13]
           LDA     '.'            ; replace non-printable characters with '.'
MD_CMD13:  CMPA    $7E
           JNC     [MD_CMD15]
           LDA     '.'
MD_CMD15:  JSR     [putchar]
; increment and check if last
           INCX
           BSTX    [TMP16]
           BLDX    [MD_ADDR]
           INCX
           BSTX    [MD_ADDR]
; compare high byte
           LDA     [MD_ADDR]
           CMPA    [MD_END]
           JC      [MD_CMD16]     ; current > end => finished
           JNZ     [MD_CMD18]
; compare low byte
           LDA     [MD_ADDR+1]
           CMPA    [MD_END+1]
           JC      [MD_CMD16]     ; start > end => finished
           JZ      [MD_CMD16]     ; start == end => finished

; line done after 16 characters
MD_CMD18:  LDA     [TMP16+1]
           AND     $0F
           JZ      [MD_CMD14]     ; line is done (HEX + ASCII)

; insert additional blank after 8 characters
           AND     $07
           JNZ     [MD_CMD12]     ; next byte
           LDA     ' '
           JSR     [putchar]
           JMP     [MD_CMD12]     ; next byte

MD_CMD14:  LDA     CR
           JSR     [putchar]
           LDA     LF
           JSR     [putchar]
; update start address
           BLDX    [MD_ADDR]
           BSTX    [MD_START]
           JMP     [MD_CMD6]      ; next line

; we are done, save address for next run
MD_CMD16:
; update start address
           BLDX    [MD_ADDR]
           BSTX    [MD_START]
           JSR     [newline]
           RTS

MD_MSG1:   .BYTE   CR, LF, "Memory display from $", 0
MD_MSG2:   .BYTE   " to $", 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function mw_cmd: execute the memory write cmd
;
; Input:  --
;
; Output: --
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
MW_CMD:    BLDX    MW_MSG1
           JSR     [puts]

; extract cmdline parameters
           BLDX    CMDLINE
           BSTX    [MW_OPT]
; skip the cmd itself
MW_CMD1:   LDA     [[MW_OPT]]
           JZ      [MW_CMD2]      ; end of string, no parameters
           BLDX    [MW_OPT]
           INCX
           BSTX    [MW_OPT]
           CMPA    ' '
           JNZ     [MW_CMD1]

; call strcpy()
           BLDX    BUF
           PUSHX
           BLDX    [MW_OPT]
           PUSHX
           JSR     [STRCPY]

; convert string to number
           BLDX    BUF
           JSR     [ATOLX]
           BSTX    [MW_START]

; get the write data
MW_CMD3:   LDA     [[MW_OPT]]
           JZ      [MW_CMD2]      ; end of string, no more parameters
           BLDX    [MW_OPT]
           INCX
           BSTX    [MW_OPT]
           CMPA    ' '
           JNZ     [MW_CMD3]

; call strcpy()
           BLDX    BUF
           PUSHX
           BLDX    [MW_OPT]
           PUSHX
           JSR     [STRCPY]

; convert string to number
           BLDX    BUF
           JSR     [ATOX]
           STA     [MW_DATA]

; get the number of bytes
MW_CMD4:   LDA     [[MW_OPT]]
           JZ      [MW_CMD2]      ; end of string, no more parameters
           BLDX    [MW_OPT]
           INCX
           BSTX    [MW_OPT]
           CMPA    ' '
           JNZ     [MW_CMD4]

; call strcpy()
           BLDX    BUF
           PUSHX
           BLDX    [MW_OPT]
           PUSHX
           JSR     [STRCPY]

; convert string to number
           BLDX    BUF
           JSR     [ATOLX]
           BSTX    [MW_NUM]
           DECX
           JZ      [MW_CMD5]      ; only one byte to write
           LDA     'e'
           JSR     [putchar]
           LDA     's'
           JSR     [putchar]
MW_CMD5:   LDA     ' '
           JSR     [putchar]
           LDA     '$'
           JSR     [putchar]

; compute end address
MW_CMD2:   LDA     [MW_START+1]
           ADD     [MW_NUM+1]
           STA     [MW_END+1]
           LDA     [MW_START]
           ADDC    [MW_NUM]
           STA     [MW_END]

; print start + end address
           LDA     [MW_START]
           JSR     [PUTHEX]
           LDA     [MW_START+1]
           JSR     [PUTHEX]
           BLDX    [MW_NUM]
           INCX
           DECX
           JZ      [MW_CMD6]      ; only 1 byte to write
           BLDX    MW_MSG2
           JSR     [puts]
           BLDX    [MW_END]
           DECX
           BSTX    [PTR16]
           LDA     [PTR16]
           JSR     [PUTHEX]
           LDA     [PTR16+1]
           JSR     [PUTHEX]
MW_CMD6:   BLDX    MW_MSG3
           JSR     [puts]
           LDA     [MW_DATA]
           JSR     [PUTHEX]
           JSR     [newline]

; now do stuff
           BLDX    0
MW_CMD7:   LDA     [MW_DATA]
           STA     [[MW_START], X]
; increment and compare
           INCX
           BSTX    [TMP16]
; compare high byte
           LDA     [MW_NUM]
           CMPA    [TMP16]
           JC      [MW_CMD7]      ; not yet done
           JZ      [MW_CMD8]      ; equal => compare low byte
           JMP     [MW_CMD9]      ; done

MW_CMD8:   LDA     [MW_NUM+1]
           CMPA    [TMP16+1]
           JC      [MW_CMD7]      ; not yet done

MW_CMD9:   JSR     [newline]
           RTS

MW_MSG1:   .BYTE   CR, LF, "Writing address", 0
MW_MSG2:   .BYTE   " to $", 0
MW_MSG3:   .BYTE   " with $", 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function load_cmd: execute the load cmd
;
; Input:  --
;
; Output: --
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
LOAD_CMD:  BLDX    LOAD_MSG1
           JSR     [puts]

           JSR     [LOAD]
           JNZ     [LD_CMD1]

           BLDX    LOAD_MSG2
           JSR     [puts]
           JMP     [LD_CMD2]

LD_CMD1:   BLDX    LOAD_MSG3
           JSR     [puts]
LD_CMD2:   RTS

LOAD_MSG1: .BYTE   CR, LF, "Waiting for Intel HEX file ", 0
LOAD_MSG2: .BYTE   CR, LF, "Success!", CR, LF, 0
LOAD_MSG3: .BYTE   CR, LF, "Error!", CR, LF, 7, 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function run_cmd: execute the run cmd
;
; Input:  --
;
; Output: --
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
RUN_CMD:   BLDX    RUN_CMD1
           JSR     [puts]
           JMP     [$4000]
           RTS

RUN_CMD1:  .BYTE   CR, LF, "Executing JMP [$4000] ... ", 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function test_cmd: put any tests here
;
; Input:  --
;
; Output: --
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
TEST_CMD:  BLDX    TEST_CMD1
           JSR     [puts]
; test the keypad
           BLDX    TEST_CMD3
           JSR     [puts]
TEST_CMD5: JSR     [KEY_GETCH]
           JZ      [TEST_CMD6]
           PUSHA
           BLDX    TEST_CMD4
           JSR     [puts]
           POPA
           JSR     [PUTHEX]
           JNZ     [TEST_CMD5]
TEST_CMD6: JSR     [newline]
           RTS

TEST_CMD1: .BYTE   "This is function TEST_CMD", CR, LF, 0
TEST_CMD2: .BYTE   "The cmdline was:", CR, LF, 0
TEST_CMD3: .BYTE   "reading keypad ... (key '0' for exit)", CR, LF, 0
TEST_CMD4: .BYTE   CR, LF, "key was: $", 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function load: load an Intel HEX file
;
; Input:  --
;
; Output: 0 .. OK
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
LOAD:      LDA     0
           STA     [ERROR]

; read a new record
LOAD1:     LDA     '.'
           JSR     [putchar]
           LDA     0
           STA     [SUM]
LOAD2:     JSR     [getch]
           CMPA    ':'
           JNZ     [LOAD2]        ; wait for start of record

; read the byte count for this record
           JSR     [LOADBYTE]
           STA     [BYTECNT]

; read the load address for this record
           JSR     [LOADBYTE]
           STA     [ADDR]
           JSR     [LOADBYTE]
           STA     [ADDR+1]

; read the record type
           JSR     [LOADBYTE]
           STA     [TYPE]
           JZ      [LOAD3]

; is end record => read checksum
           JSR     [LOADBYTE]
           STA     [CHECKSUM]
           LDA     [SUM]
           JZ      [LOADRTN]      ; OK
           LDA     [ERROR]
           INCA
           STA     [ERROR]
           JMP     [LOADRTN]

; is data record
LOAD3:     BLDX    0
LOAD4:     JSR     [LOADBYTE]
           STA     [[ADDR], X]
           LDA     [BYTECNT]
           DECA
           JZ      [LOAD5]        ; last byte of this record
           STA     [BYTECNT]
           INCX
           JMP     [LOAD4]

; read the check sum for this record
LOAD5:     JSR     [LOADBYTE]
           STA     [CHECKSUM]
           LDA     [SUM]
           JZ      [LOAD1]        ; OK => next record
           LDA     [ERROR]
           INCA
           STA     [ERROR]
           JMP     [LOAD1]        ; error => next record

LOADRTN:   JSR     [getch]
           LDA     [ERROR]
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function loadbyte: recieve two hex characters and form a byte
;                    add to SUM
;
; Input:  --
;
; Output: byte in ACC
;         added to SUM
;
; Uses:   A, Z, SUM, TMP
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
LOADBYTE:  JSR     [getch]
           CMPA    '9'
           JC      [LDBYTE1]
           SUB     '0'
           JMP     [LDBYTE2]
LDBYTE1:   OR      $20
           SUB     'a'-10
LDBYTE2:   SHL
           SHL
           SHL
           SHL
           STA     [TMP]
           JSR     [getch]
           CMPA    '9'
           JC      [LDBYTE3]
           SUB     '0'
           JMP     [LDBYTE4]
LDBYTE3:   OR      $20
           SUB     'a'-10
LDBYTE4:   OR      [TMP]
           STA     [TMP]
           ADD     [SUM]
           STA     [SUM]
           LDA     [TMP]
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function prompt: output a newline and a prompt to the RS-232
;
; Input:  --
;
; Output: --
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PROMPT:    LDA     CR
           JSR     [putchar]
           LDA     LF
           JSR     [putchar]
           LDA     '#'
           JSR     [putchar]
           LDA     ' '
           JSR     [putchar]
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function newline: output a CR to the RS-232
;
; Input:  --
;
; Output: --
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
newline:   LDA     CR
           JSR     [putchar]
           LDA     LF
           JSR     [putchar]
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function key_getch: read a character from the keypad
;
; Input:  --
;
; Output: the keycode in A
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
KEY_GETCH: LDA     [KEYPAD]
           JN      [KEY_GETCH]    ; if bit7 is set, no key available
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function LCDPUT: put a character to the LCD
;
; Input:  A .. the character
;
; Output: the character
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
LCDPUT:    STA     [MAINDISP]     ; put the character into the LCD
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function LCDPUTS: put a string to the LCD
; Input:  X .. address of string
;
; Output: number of characters written
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
LCDPUTS:   BSTX    [PTR16]        ; save address of string
           BLDX    0              ; load the index register with 0 
LCDPUTS1:  LDA     [[PTR16],X]    ; load accumulator with character
           JZ      [LCDPUTS2]     ; end of string => return
           STA     [MAINDISP]     ; write character to UART
           INCX
           JMP     [LCDPUTS1]     ; next character
LCDPUTS2:  BSTX    [PTR16]
           DECA
           LDA     [PTR16]        ; get number of written bytes
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function LCDPUTH: put a HEX byte to the LCD
;
; Input: A .. the digit
;
; Output: --
;
; Uses:  A, Z, Carry
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
LCDPUTH:   PUSHA
; process upper nibble
           SHR
           SHR
           SHR
           SHR
           AND     $0F
           CMPA    9
           JC      [LCDPUTH1]
           ADD     '0'
           JMP     [LCDPUTH2]
LCDPUTH1:  ADD     'A'
           SUB     10
LCDPUTH2:  STA     [MAINDISP]    ; write to LCD

; process lower nibble
           POPA
           AND     $0F
           CMPA    9
           JC      [LCDPUTH3]
           ADD     '0'
           JMP     [LCDPUTH4]
LCDPUTH3:  ADD     'A'
           SUB     10
LCDPUTH4:  STA     [MAINDISP]    ; write to LCD
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function atox: convert a string to an 8bit integer, interpret as HEX
;
; Input:  X .. ptr to string
;
; Output: A .. the 8bit number
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ATOX:      BSTX    [PTR16]        ; save the ptr
           BLDX    0
; the 1st digit
           LDA     [[PTR16],X]    ; get hex digit
           JZ      [ATOX5]        ; end of input string
           CMPA    '9'
           JC      [ATOX1]
           SUB     '0'
           JMP     [ATOX2]
ATOX1:     OR      $20            ; "tolower"
           SUB     'a'-10
ATOX2:     STA     [TMP]

; the 2nd digit
           INCX
           LDA     [[PTR16],X]    ; get hex digit
           JZ      [ATOX6]        ; end of input string
           CMPA    '9'
           JC      [ATOX3]
           SUB     '0'
           JMP     [ATOX4]
ATOX3:     OR      $20            ; "tolower"
           SUB     'a'-10
ATOX4:     PUSHA
           LDA     [TMP]
           SHL
           SHL
           SHL
           SHL
           STA     [TMP]
           POPA
ATOX6:     OR      [TMP]
ATOX5:     RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function atolx: convert a string to a 16 bit integer, interpret as HEX
;
; Input:  X .. ptr to string
;
; Output: X .. the 16bit number
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ATOLX:     BSTX    [PTR16]        ; save the ptr
           BLDX    0
; the 1st digit
           LDA     [[PTR16],X]    ; get hex digit
           JZ      [ATOLX5]       ; end of input string
           CMPA    '9'
           JC      [ATOLX1]
           SUB     '0'
           JMP     [ATOLX2]
ATOLX1:    OR      $20            ; "tolower"
           SUB     'a'-10
ATOLX2:    STA     [TMP]

; the 2nd digit
           INCX
           LDA     [[PTR16],X]    ; get hex digit
           JZ      [ATOLX6]
; end of input string
           CMPA    '9'
           JC      [ATOLX3]
           SUB     '0'
           JMP     [ATOLX4]
ATOLX3:    OR      $20            ; "tolower"
           SUB     'a'-10
ATOLX4:    PUSHA
           LDA     [TMP]
           SHL
           SHL
           SHL
           SHL
           STA     [TMP]
           POPA
           OR      [TMP]
           STA     [TMP]
           STA     [TMP16]

; the 3rd digit
           INCX
           LDA     [[PTR16],X]    ; get hex digit
           JZ      [ATOLX6]       ; end of input string
           CMPA    '9'
           JC      [ATOLX7]
           SUB     '0'
           JMP     [ATOLX8]
ATOLX7:    OR      $20            ; "tolower"
           SUB     'a'-10
ATOLX8:    STA     [TMP]

; the 4th digit
           INCX
           LDA     [[PTR16],X]    ; get hex digit
           JZ      [ATOLX11]      ; end of input string
           CMPA    '9'
           JC      [ATOLX9]
           SUB     '0'
           JMP     [ATOLX10]
ATOLX9:    OR      $20            ; "tolower"
           SUB     'a'-10
ATOLX10:   PUSHA
           LDA     [TMP]
           SHL
           SHL
           SHL
           SHL
           STA     [TMP]
           POPA
           OR      [TMP]
           STA     [TMP16+1]
           BLDX    [TMP16]
           RTS

; empty string => return 0
ATOLX5:    BLDX    0
           RTS

; one or two digits => save number to low byte
ATOLX6:    LDA     [TMP]
           STA     [TMP16+1]
           LDA     0
           STA     [TMP16]
           BLDX    [TMP16]
           RTS

; 3 digits
ATOLX11:   LDA     [TMP]
           STA     [TMP16+1]
; put low digit of high byte to high digit of low byte
           LDA     [TMP16]
           AND     $0F
           SHL
           SHL
           SHL
           SHL
           OR      [TMP16+1]
           STA     [TMP16+1]
; put high digit of high byte to low digit of high byte
           LDA     [TMP16]
           SHR
           SHR
           SHR
           SHR
           AND     $0F
           STA     [TMP16]
           BLDX    [TMP16]
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function strcpy: copy one string to another
;
; Input:  two 16bit pointers on the stack
;
; Output: X .. ptr to destination string
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
STRCPY:    POPA
           STA     [R_ADD]        ; save MSB of return address
           POPA
           STA     [R_ADD+1]      ; save LSB of return address

           POPA
           STA     [S2]           ; save LSB of src
           POPA
           STA     [S2+1]         ; save MSB of src
           POPA
           STA     [S1]           ; save LSB of dest
           POPA
           STA     [S1+1]         ; save MSB of dest
           BLDX    $FFFF

STRCPY1:   INCX
           LDA     [[S2], X]
           STA     [[S1], X]
           JZ      [STRCPY2]      ; end of src
           JMP     [STRCPY1]      ; next character

; src reached end
STRCPY2:   LDA     [R_ADD+1]
           PUSHA                  ; restore LSB of return address
           LDA     [R_ADD]
           PUSHA                  ; restore MSB of return address
           BLDX    [S1]           ; put result into X
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function strcmp: compare two null terminated strings
;
; Input:  two 16bit pointers on the stack
;
; Output: 0 ... s1 == s2
;         -1 .. s1 < s2
;         1 ... s1 > s2
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
STRCMP:    POPA
           STA     [R_ADD]        ; save MSB of return address
           POPA
           STA     [R_ADD+1]      ; save LSB of return address

           POPA
           STA     [S2]           ; save LSB of s2
           POPA
           STA     [S2+1]         ; save MSB of s2
           POPA
           STA     [S1]           ; save LSB of s1
           POPA
           STA     [S1+1]         ; save MSB of s1
           BLDX    $FFFF

STRCMP1:   INCX
           LDA     [[S2], X]
           JZ      [STRCMP2]      ; end of s2
           STA     [TMP]
           LDA     [[S1], X]
           JZ      [STRCMP3]      ; end of s1 but not of s2 => s2 > s1
           CMPA    [TMP]
           JZ      [STRCMP1]      ; equal => next character
           JC      [STRCMP4]      ; s1 > S2
STRCMP3:   LDA     $FF
           JMP     [STRCMP5]
STRCMP4:   LDA     1
           JMP     [STRCMP5]

; s2 reached end
STRCMP2:   LDA     [[S1], X]
           JNZ     [STRCMP4]      ; s1 not at end => s1 > s2
           LDA     0
STRCMP5:   STA     [TMP]
STRCMP6:   LDA     [R_ADD+1]
           PUSHA                  ; restore LSB of return address
           LDA     [R_ADD]
           PUSHA                  ; restore MSB of return address
           LDA     [TMP]          ; put result into acc
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function PUTHEX: put a HEX byte to the RS-232
;
; Input: A .. the byte
;
; Output: the byte in A
;
; Uses:  A, Z, Carry
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PUTHEX:    PUSHA
           PUSHA
; process upper nibble
           SHR
           SHR
           SHR
           SHR
           AND     $0F
           CMPA    9
           JC      [PUTHEX1]
           ADD     '0'
           JMP     [PUTHEX2]
PUTHEX1:   ADD     'A'-10
PUTHEX2:   JSR     [putchar]

; process lower nibble
           POPA
           AND     $0F
           CMPA    9
           JC      [PUTHEX3]
           ADD     '0'
           JMP     [PUTHEX4]
PUTHEX3:   ADD     'A'-10
PUTHEX4:   JSR     [putchar]
           POPA
           RTS

;===================================================================
; global functions
;===================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function PUTCHAR: put a character to the RS-232
;
; Input:  A .. the character
;
; Output: the character
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_PUTCHAR:  PUSHA
PUTCHAR1:  LDA     [UART_STA]     ; read UART status
           AND     UART_TXE
           JZ      [PUTCHAR1]     ; wait for transmitter empty
           POPA                   ; transmitter is empty now, get the char
           STA     [UART_TX]      ; ... and put it into the UART
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function PUTS: put a string to the RS-232
;
; Input:  X .. address of string
;
; Output: number of characters written
;
; Uses:   A, X, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_PUTS:     BSTX    [PTR16]        ; save address of string
           BLDX    0              ; load the index register with 0 
PUTS1:     LDA     [UART_STA]     ; read UART status
           AND     UART_TXE
           JZ      [PUTS1]        ; wait for transmitter empty
           LDA     [[PTR16],X]    ; load accumulator with character
           JZ      [PUTS2]        ; end of string => return
           STA     [UART_TX]      ; write character to UART
           INCX
           JMP     [PUTS1]        ; next character
PUTS2:     BSTX    [PTR16]
           DECA
           LDA     [PTR16]        ; get number of transmitted bytes
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function TSTC: test if a charcter is available from the RS-232
;
; Input:  --
;
; Output: 0 .. no character available
;         1 .. character available
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_TSTC:     LDA     [UART_STA]     ; read UART status
           AND     UART_RXR
           JZ      [TSTC1]        ; no byte ready
           LDA     1              ; load 1 into acc
TSTC1      RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function GETCH: read a character from the RS-232
;
; Input:  --
;
; Output: the character
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_GETCH:    LDA     [UART_STA]     ; read UART status
           AND     UART_RXR
           JZ      [_GETCH]       ; wait for byte available
           LDA     [UART_RX]      ; read the byte from the UART
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function GETCHE: read a character from the RS-232 with echo
;
; Input:  --
;
; Output: the character
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_GETCHE:   LDA     [UART_STA]     ; read UART status
           AND     UART_RXR
           JZ      [_GETCHE]      ; wait for byte available
           LDA     [UART_RX]      ; read the byte from the UART
; echo the character
           PUSHA
GETCHE1:   LDA     [UART_STA]
           AND     UART_TXE
           JZ      [GETCHE1]      ; wait for transmitter free
           POPA
           STA     [UART_TX]      ; write to UART
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function GETCHAR: read a char from the RS-232 and wait for Enter
;
; Input:  --
;
; Output: the character
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_GETCHAR:  LDA     [UART_STA]     ; read UART status
           AND     UART_RXR
           JZ      [_GETCHAR]     ; wait for byte available
           LDA     [UART_RX]      ; read the byte from the UART
           PUSHA                  ; save character
; echo the character
GETCHAR1:  PUSHA
GETCHAR2:  LDA     [UART_STA]
           AND     UART_TXE
           JZ      [GETCHAR2]     ; wait for transmitter free
           POPA
           STA     [UART_TX]      ; write to UART
GETCHAR3:  LDA     [UART_STA]     ; read UART status
           AND     UART_RXR
           JZ      [GETCHAR3]     ; wait for byte available
           LDA     [UART_RX]      ; read the byte from the UART
           CMPA    CR
           JNZ     [GETCHAR1]     ; wait for carriage return
           PUSHA
GETCHAR4:  LDA     [UART_STA]
           AND     UART_TXE
           JZ      [GETCHAR4]     ; wait for transmitter free
           POPA
           STA     [UART_TX]      ; write to UART
           POPA                   ; restore character
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function GETS: read a line from the RS-232
;
; Input:  X .. address of string
;
; Output: X .. address of string
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_GETS:     BSTX    [PTR16]        ; save address of string
           BLDX    0              ; load the index register with 0 
GETS1:     LDA     [UART_STA]     ; read UART status
           AND     UART_RXR
           JZ      [GETS1]        ; wait for byte available
           LDA     [UART_RX]      ; read the byte from the UART
; backspace support
           CMPA    BS
           JNZ     [GETS2]
           BSTX    [TMP16]
           LDA     [TMP16+1]
           JZ      [GETS1]        ; dont write over the left end
GETS4:     LDA     [UART_STA]
           AND     UART_TXE
           JZ      [GETS4]        ; wait for transmitter free
           LDA     BS
           STA     [UART_TX]      ; write to UART
GETS5:     LDA     [UART_STA]
           AND     UART_TXE
           JZ      [GETS5]        ; wait for transmitter free
           LDA     ' '            ; overwrite with ' '
           STA     [UART_TX]      ; write to UART
           DECX                   ; decrement index
GETS6:     LDA     [UART_STA]
           AND     UART_TXE
           JZ      [GETS6]        ; wait for transmitter free
           LDA     BS
           STA     [UART_TX]      ; write to UART
           JMP     [GETS1]        ; next character

; echo the character
GETS2:     PUSHA
GETS3:     LDA     [UART_STA]
           AND     UART_TXE
           JZ      [GETS3]        ; wait for transmitter free
           POPA
           STA     [UART_TX]      ; write to UART

; check for line end
           CMPA    CR
           JZ      [GETS7]        ; reached the end
           CMPA    LF
           JZ      [GETS7]        ; reached the end
           STA     [[PTR16],X]    ; save character
           INCX
           JMP     [GETS1]        ; next character

GETS7:     LDA     0
           STA     [[PTR16],X]    ; save the terminating '\0' character

; output a newline
GETS8:     LDA     [UART_STA]     ; read UART status
           AND     UART_TXE
           JZ      [GETS8]        ; wait for transmitter empty
           LDA     CR
           STA     [UART_TX]      ; put CR into the UART

GETS9:     LDA     [UART_STA]     ; read UART status
           AND     UART_TXE
           JZ      [GETS9]        ; wait for transmitter empty
           LDA     LF
           STA     [UART_TX]      ; put LF into the UART

           BLDX    [PTR16]        ; restore X
           RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Function NOTYET: print a message the this function is not yet implemented
;
; Input:  --
;
; Output: --
;
; Uses:   A, Z
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_NOTYET:   BLDX    NOTYETMSG
           JSR     [_PUTS]
           JSR     [_GETCH]
           LDA     CR
           JSR     [_PUTCHAR]
           LDA     LF
           JSR     [_PUTCHAR]
           RTS

NOTYETMSG: .BYTE   "This function is not yet implemented",CR,LF,"press any key ... ",0

           .END
